<?php

namespace Barn2\Plugin\WC_Restaurant_Ordering;

use Barn2\WRO_Lib\Admin\Notices;
use Barn2\WRO_Lib\Plugin\Licensed_Plugin;
use Barn2\WRO_Lib\Plugin\Premium_Plugin;
use Barn2\WRO_Lib\Registerable;
use Barn2\WRO_Lib\Service_Container;
use Barn2\WRO_Lib\Service_Provider;
use Barn2\WRO_Lib\Translatable;
use Barn2\WRO_Lib\Util;

/**
 * The main plugin class. Responsible for setting up to core plugin services.
 *
 * @package   Barn2\woocommerce-restaurant-ordering
 * @author    Barn2 Plugins <support@barn2.com>
 * @license   GPL-3.0
 * @copyright Barn2 Media Ltd
 */
class Plugin extends Premium_Plugin implements Registerable, Translatable, Service_Provider, Licensed_Plugin {

	use Service_Container;

	const NAME    = 'WooCommerce Restaurant Ordering';
	const ITEM_ID = 236796;

	/**
	 * Constructs and initialize the main plugin class.
	 *
	 * @param string $file    The path to the main plugin file.
	 * @param string $version The current plugin version.
	 */
	public function __construct( $file = null, $version = '1.0' ) {
		parent::__construct(
			[
				'name'               => self::NAME,
				'item_id'            => self::ITEM_ID,
				'version'            => $version,
				'file'               => $file,
				'is_woocommerce'     => true,
				'settings_path'      => 'admin.php?page=wc-settings&tab=restaurant-ordering',
				'documentation_path' => 'kb-categories/wro-kb'
			]
		);
	}

	/**
	 * Registers the plugin with WordPress.
	 */
	public function register() {
		parent::register();

		// Create Plugin_Setup here so the activation hook will run.
		$plugin_setup = new Admin\Plugin_Setup( $this->get_file(), $this );
		$plugin_setup->register();

		add_action( 'plugins_loaded', [ $this, 'maybe_load_plugin' ] );
	}

	public function maybe_load_plugin() {
		// Don't load anything if WooCommerce not active.
		if ( ! Util::is_woocommerce_active() ) {
			$this->add_missing_woocommerce_notice();
			return;
		}

		add_action( 'init', [ $this, 'load_textdomain' ], 5 );
		add_action( 'init', [ $this, 'register_services' ] );
	}

	public function get_services() {
		$services = [
			'admin'          => new Admin\Admin_Controller( $this ),
			'plugin_upgrade' => new Admin\Plugin_Upgrade_Check( $this )
		];

		if ( ! $this->has_valid_license() ) {
			return $services;
		}

		$rest_controller = new Rest\Rest_Controller();

		return array_merge(
			$services,
			[
				'shortcodes'       => new Shortcodes(),
				'frontend_scripts' => new Frontend_Scripts( $this, $rest_controller ),
				'rest_controller'  => $rest_controller,
				'cart_handler'     => new Cart\Cart_Validation(),
				'template_handler' => new Template_Handler(),
				'product_addons'   => new Integration\Product_Addons(),
				'product_table'    => new Integration\Product_Table()
			]
		);
	}

	public function load_textdomain() {
		load_plugin_textdomain( 'woocommerce-restaurant-ordering', false, $this->get_slug() . '/languages' );
	}

	private function add_missing_woocommerce_notice() {
		if ( Util::is_admin() ) {
			$admin_notice = new Notices();
			$admin_notice->add(
				'wro_woocommerce_missing',
				'',
				sprintf(
				/* translators: %1$s: open link tag, %2$s: close link tag, %3$s: the plugin name */
					__( 'Please %1$sinstall WooCommerce%2$s in order to use the %3$s extension.', 'woocommerce-restaurant-ordering' ),
					Util::format_link_open( 'https://woocommerce.com/', true ),
					'</a>',
					$this->get_name()
				),
				[
					'type'       => 'error',
					'capability' => 'install_plugins'
				]
			);
			$admin_notice->boot();
		}
	}

}
